package se.sveaekonomi.webpay.integration.hosted.helper;

import java.io.ByteArrayOutputStream;
import java.io.UnsupportedEncodingException;
import java.util.ArrayList;
import java.util.List;

import javax.xml.stream.XMLOutputFactory;
import javax.xml.stream.XMLStreamException;

import se.sveaekonomi.webpay.integration.exception.SveaWebPayException;
import se.sveaekonomi.webpay.integration.hosted.HostedOrderRowBuilder;
import se.sveaekonomi.webpay.integration.hosted.payment.HostedPayment;
import se.sveaekonomi.webpay.integration.order.create.CreateOrderBuilder;
import se.sveaekonomi.webpay.integration.order.identity.CompanyCustomer;
import se.sveaekonomi.webpay.integration.order.identity.CustomerIdentity;
import se.sveaekonomi.webpay.integration.order.identity.IndividualCustomer;
import se.sveaekonomi.webpay.integration.util.xml.XMLBuilder;

public class HostedXmlBuilder extends XMLBuilder {

    public String getXml(HostedPayment<?> payment) {
        XMLOutputFactory xmlof = XMLOutputFactory.newInstance();
        ByteArrayOutputStream os = new ByteArrayOutputStream();
        CreateOrderBuilder order = payment.getCreateOrderBuilder();
        ArrayList<HostedOrderRowBuilder> rows = payment.getRowBuilder();
        
        try {
            xmlw = xmlof.createXMLStreamWriter(os, "UTF-8");
            xmlw.writeStartDocument("UTF-8", "1.0");
            xmlw.writeComment("Message generated by Integration package Java");// V + Config.version);
            xmlw.writeStartElement("payment");
            xmlw = payment.getPaymentSpecificXml(xmlw);
            
            writeSimpleElement("customerrefno", order.getClientOrderNumber());
            writeSimpleElement("currency", order.getCurrency());
            writeSimpleElement("amount", payment.getAmount().toString());
            
            if (payment.getVat() != null) {
                writeSimpleElement("vat", payment.getVat().toString());
            }
            
            if (payment.getPayPageLanguageCode() != null) {
                writeSimpleElement("lang", payment.getPayPageLanguageCode());
            }
            
            writeSimpleElement("returnurl", payment.getReturnUrl());
            writeSimpleElement("cancelurl", payment.getCancelUrl());
            writeSimpleElement("iscompany", order.getIsCompanyIdentity() ? "true" : "false");
            
            serializeCustomer(order, payment);
            serializeRows(rows);
            serializeExcludedPaymentMethods(payment.getExcludedPaymentMethods());
            
            writeSimpleElement("addinvoicefee", "false");
            xmlw.writeEndDocument();
            
            return new String(os.toByteArray(), "UTF-8");
        } catch (XMLStreamException e) {
            throw new SveaWebPayException("Error when building XML", e);
        } catch (UnsupportedEncodingException e) {
            throw new SveaWebPayException("Unsupported encoding UTF-8", e);
        } catch (Exception e) {
            throw new SveaWebPayException("Exception", e);
        }
    }
    
    private void serializeCustomer(CreateOrderBuilder order, HostedPayment<?> payment) {
        CustomerIdentity<?> customer;
        
        try {
            if (order.getIsCompanyIdentity()) {
                customer = order.getCompanyCustomer();
            } else {
                customer = order.getIndividualCustomer();
            }
            
            xmlw.writeStartElement("customer");
            
            //Nordic country individual customer type
            if (customer.getNationalIdNumber() != null) {
                writeSimpleElement("ssn", customer.getNationalIdNumber());
            }
            //Euro country individual
            else if (!order.getIsCompanyIdentity()) {
                writeSimpleElement("ssn", ((IndividualCustomer)customer).getBirthDate().toString());
            }
            //Euro country, company customer and nationalId not set
            else if (order.getIsCompanyIdentity()) {
                writeSimpleElement("ssn", ((CompanyCustomer)customer).getVatNumber());
            }
            
            //Set for individual customer
            if (!order.getIsCompanyIdentity()) {
                IndividualCustomer individualCustomer = (IndividualCustomer)customer;
                
                if (individualCustomer.getFirstName() != null) {
                    writeSimpleElement("firstname", individualCustomer.getFirstName());
                }
                
                if (individualCustomer.getLastName() != null) {
                    writeSimpleElement("lastname", individualCustomer.getLastName());
                }
                
                if (individualCustomer.getInitials() != null) {
                    writeSimpleElement("initials", individualCustomer.getInitials());
                }
            } else { //Set for company customer
                CompanyCustomer companyCustomer = (CompanyCustomer)customer;
                
                if (companyCustomer.getCompanyName() != null) {
                    writeSimpleElement("firstname", companyCustomer.getCompanyName());
                }
            }
            
            if (customer.getPhoneNumber() != null) {
                writeSimpleElement("phone", customer.getPhoneNumber());
            }
            
            if (customer.getEmail() != null) {
                writeSimpleElement("email", customer.getEmail());
            }
            
            if (customer.getStreetAddress() != null) {
                writeSimpleElement("address", customer.getStreetAddress());
            }
            
            if (customer.getHouseNumber() != null) {
                writeSimpleElement("housenumber", customer.getHouseNumber());
            }
            
            if (customer.getCoAddress() != null) {
                writeSimpleElement("address2", customer.getCoAddress());
            }
            
            if (customer.getZipCode() != null) {
                writeSimpleElement("zip", customer.getZipCode());
            }
            
            if (customer.getLocality() != null) {
                writeSimpleElement("city", customer.getLocality().toString());
            }
            
            if (order.getCountryCode() != null) {
                writeSimpleElement("country", order.getCountryCode().toString());
            }
            
            xmlw.writeEndElement();
            
            if (order.getIsCompanyIdentity()) {
                if (order.getCompanyCustomer().getIpAddress() != null) {
                    writeSimpleElement("ipaddress", order.getCompanyCustomer().getIpAddress());
                }
            } else {
                if (order.getIndividualCustomer().getIpAddress() != null) {
                    writeSimpleElement("ipaddress", order.getIndividualCustomer().getIpAddress());
                }
            }
        } catch (XMLStreamException e) {
            throw new SveaWebPayException("Error while building XML", e);
        }
    }
    
    private void serializeRows(List<HostedOrderRowBuilder> rows) throws XMLStreamException {
        if (rows == null || rows.size() == 0) {
            return;
        }
        
        xmlw.writeStartElement("orderrows");
        
        for (HostedOrderRowBuilder row : rows) {
            serializeRow(row);
        }
        
        xmlw.writeEndElement();
    }
    
    private void serializeRow(HostedOrderRowBuilder row) throws XMLStreamException {
        xmlw.writeStartElement("row");
        
        writeSimpleElement("sku", row.getSku());
        writeSimpleElement("name", row.getName());
        writeSimpleElement("description", row.getDescription());
        
        if (row.getAmount() != null) {
            writeSimpleElement("amount", row.getAmount().toString());
        }
        
        if (row.getVat() != null) {
            writeSimpleElement("vat", row.getVat().toString());
        }
        
        if (row.getQuantity() != 0) {
            writeSimpleElement("quantity", Integer.toString(row.getQuantity()));
        }
        
        writeSimpleElement("unit", row.getUnit());
        
        xmlw.writeEndElement();
    }
    
    private void serializeExcludedPaymentMethods(List<String> excludedPaymentMethods) throws XMLStreamException {
        if (excludedPaymentMethods != null && !excludedPaymentMethods.isEmpty()) {
            xmlw.writeStartElement("excludepaymentmethods");
            
            List<String> excludeList =  excludedPaymentMethods;
            
            for (int i = 0; i < excludeList.size(); i++) {
                writeSimpleElement("exclude", excludeList.get(i));
            }
            
            xmlw.writeEndElement();
        }
    }
}
